import firebase_admin
from django.conf import settings
from django.test.signals import setting_changed

FIREBASE_CREDENTIALS_FILE = getattr(settings, "FIREBASE_CREDENTIALS_FILE", None)
FIREBASE_CREDENTIALS_DICT = getattr(settings, "FIREBASE_CREDENTIALS_DICT", None)

# Initialize Firebase Admin SDK
if FIREBASE_CREDENTIALS_FILE or FIREBASE_CREDENTIALS_DICT:
    cert = FIREBASE_CREDENTIALS_FILE or FIREBASE_CREDENTIALS_DICT
    firebase_admin.initialize_app(
        credential=firebase_admin.credentials.Certificate(cert=cert)
    )
else:
    try:
        firebase_admin.initialize_app()
    except ValueError as e:
        raise ValueError(
            "Firebase credentials not found\n "
            "set FIREBASE_CREDENTIALS_FILE or FIREBASE_CREDENTIALS_JSON in django settings"
        ) from e


def reload_settings(*args, **kwargs):
    setting_changed.send(
        sender=__name__,
        setting="FIREBASE_CREDENTIALS_FILE",
        value=FIREBASE_CREDENTIALS_FILE,
    )
    setting_changed.send(
        sender=__name__,
        setting="FIREBASE_CREDENTIALS_DICT",
        value=FIREBASE_CREDENTIALS_DICT,
    )


setting_changed.connect(reload_settings)



from django.contrib.auth.models import UserManager as DefaultUserManager
from firebase_admin import auth


class UserManager(DefaultUserManager):
    def _create_user(self, email, password, **extra_fields):
        email = self.normalize_email(email)
        phone_number = extra_fields.get("phone_number", None)
        display_name = extra_fields.get("display_name", None)
        auth_user = auth.create_user(
            email=email,
            email_verified=False,
            password=password,
            disabled=False,
            **phone_number and {"phone_number": str(phone_number)},
            **display_name and {"display_name": display_name},
        )
        user = self.model(id=auth_user.uid, email=email, **extra_fields)
        user.save(using=self._db)
        return user

    def create_user(self, email, password=None, **extra_fields):
        extra_fields.setdefault("is_staff", False)
        extra_fields.setdefault("is_superuser", False)
        return self._create_user(email, password, **extra_fields)

    def create_superuser(self, email, password=None, **extra_fields):
        extra_fields.setdefault("is_staff", True)
        extra_fields.setdefault("is_superuser", True)

        if extra_fields.get("is_staff") is not True:
            raise ValueError("Superuser must have is_staff=True.")
        if extra_fields.get("is_superuser") is not True:
            raise ValueError("Superuser must have is_superuser=True.")

        return self._create_user(email, password, **extra_fields)



from django.db import models


class UserFirebaseProfile(models.Model):
    user = models.OneToOneField("auth.User", on_delete=models.CASCADE)
    display_name = models.CharField(max_length=255, blank=True, null=True)
    phone_number = models.CharField(max_length=255, blank=True, null=True)
    photo_url = models.CharField(max_length=255, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    uid = models.CharField(max_length=255, blank=True, null=True)



"""
Handle the authentication of the user.
Using google authentication (with application)
Using firebase authentication (with web)
"""
import datetime

import firebase_admin.auth
import httplib2
from django.conf.global_settings import DEBUG
from django.contrib.auth.models import User
from django.utils import timezone
from google.auth import jwt
from google.oauth2.id_token import verify_oauth2_token
from google_auth_httplib2 import Request
from rest_framework import authentication, status
from rest_framework.exceptions import APIException

from django_firebase_auth.models import UserFirebaseProfile


class NoAuthToken(APIException):
    status_code = status.HTTP_401_UNAUTHORIZED
    default_detail = "No authentication token provided"
    default_code = "no_auth_token"


class InvalidAuthToken(APIException):
    status_code = status.HTTP_401_UNAUTHORIZED
    default_detail = "Invalid authentication token provided"
    default_code = "invalid_token"


class TokenExpired(APIException):
    status_code = status.HTTP_401_UNAUTHORIZED
    default_detail = "Authorization token is expired"
    default_code = "token_expired"


class FirebaseError(APIException):
    status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
    default_detail = (
        "The user provided with the auth token is not a "
        "valid Firebase user, it has no Firebase UID"
    )
    default_code = "no_firebase_uid"


def auth_with_firebase(id_token):
    try:
        decoded_token = firebase_admin.auth.verify_id_token(id_token)
    except ValueError as e:
        raise InvalidAuthToken() from e
    return decoded_token


def auth_with_application(id_token, decoded_token):
    try:
        decoded_token = verify_oauth2_token(
            id_token,
            request=Request(
                http=httplib2.Http(),
            ),
            audience=decoded_token["aud"],
        )

    except ValueError as e:
        raise InvalidAuthToken() from e
    return decoded_token


class FirebaseAuthentication(authentication.BaseAuthentication):
    def authenticate(self, request):
        auth_header = request.META.get("HTTP_AUTHORIZATION")
        if not auth_header:
            raise InvalidAuthToken("Authorization header not present")
        host = request.get_host()
        if (
            host.startswith("localhost")
            and not auth_header.startswith("Bearer ")
            and DEBUG
        ):
            return User.objects.get(username=auth_header), None
        id_token = auth_header.split(" ").pop()
        try:
            decoded_token = jwt.decode(id_token, verify=False)
        except ValueError as e:
            raise InvalidAuthToken() from e
        is_expired = (
            datetime.datetime.fromtimestamp(decoded_token["exp"])
            < datetime.datetime.now()
        )
        if is_expired:
            raise InvalidAuthToken("Authorization token is expired")
        if "firebase" in decoded_token:
            decoded_token = auth_with_firebase(id_token)
        else:
            decoded_token = auth_with_application(id_token, decoded_token)

        if not id_token or not decoded_token:
            return None

        striped_user_name = decoded_token["email"].split("@")[0]
        display_name = decoded_token.get("name", striped_user_name)
        first_name, last_name = self.convert_user_display_name(display_name)
        user: User = User.objects.get_or_create(
            email=decoded_token.get("email"),
            defaults={
                "username": striped_user_name,
                "first_name": first_name,
                "last_name": last_name,
            },
        )[0]
        profile: UserFirebaseProfile = UserFirebaseProfile.objects.get_or_create(
            user=user, uid=decoded_token.get("uid")
        )[0]

        if decoded_token.get("picture"):
            profile.photo_url = decoded_token.get("picture")
        if decoded_token.get("phone_number"):
            profile.phone_number = decoded_token.get("phone_number")
        if decoded_token.get("name"):
            profile.display_name = decoded_token.get("name")
        profile.save()
        user.last_login = timezone.now()
        profile.save()
        user.save(update_fields=["last_login"])

        return user, None

    def convert_user_display_name(self, display_name: str):
        """
        Convert user display name to first name and last name
        :arg:
        display_name(str): user display name
        :return:
        first_name(str): first name of user
        last_name(str): last name of user
        """
        names = display_name.split(" ")
        first_name = names[0]
        last_name = None
        if len(names) > 1:
            last_name = names[1]
        return first_name, last_name



import os
from pathlib import Path

from setuptools import find_packages, setup

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="django-firebase-auth",
    version="1.0.7",
    packages=find_packages(),
    install_requires=["firebase-admin", "djangorestframework"],
    url="https://github.com/maycuatroi/django-firebase-auth",
    include_package_data=True,
    license="MIT License",
    description="Django authentication middle ware using Firebase Authentication Service",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Nguyen Anh Binh",
    author_email="sometimesocrazy@gmail.com",
)
